import QtQuick 2.0
import MuseScore 3.0


MuseScore {
    version: "1.0"
    description: "Fill voices in a selection with rests"
    menuPath: "Plugins.Waltz2Swing."

    id: waltz2Swing

    Component.onCompleted : {
        if (mscoreMajorVersion >= 4) {
            Waltz2Swing.title = "From Waltz to Swing"
        }
     }


    // create and return a new Note element with given pitch, tpc
	function createNote(pitch, tpc){
		//console.log("pitch = ", pitch, "tpc = ", tpc)
        var note = newElement(Element.NOTE);
        note.pitch = pitch;
		note.tpc = tpc;
        note.tpc1 = tpc;
        note.tpc2 = tpc;
        return note;
    }// end CreateNote

    function setCursorToTime(cursor, time){
        cursor.rewind(0);
        while (cursor.segment) { 
            var current_time = cursor.tick;
            if(current_time>=time){
                return true;
            }
            cursor.next();
        }
        cursor.rewind(0);
        return false;
    }// end setcursor To Time

    function smartQuit() {
        if (mscoreMajorVersion < 4) {
            Qt.quit()
        } else {
            quit()
        }
    } //smartQuit

	function collectchordnames(segment, chordnames,count){
    while (segment) {
        var aCount = 0;
        var annotation = segment.annotations[aCount];

        while (annotation) {
            if (annotation.type == Element.HARMONY) {
                var harmony =  annotation;
                var str = harmony.text;
                /*
                //if (str != "")
                //    str = correctCase(str);
                var r = str.indexOf("Maj");
                if(str.substr(str.length-3,3)== "Maj"){
                    str = str.substr(0,str.length-3)
                    console.log('Maj string = ' + str);
                }
                */
                chordnames[count][0] = str;
                chordnames[count][1] = segment.tick;
                
                // avoid spurious empty harmonies
                if (chordnames[count][0] != ""){
                    count++;
                }					    
            }
            annotation = segment.annotations[++aCount];
        }
        segment = segment.next;
    }
    return count;
}

    onRun: {

    var tpc2note = ["cbb","gbb","dbb","abb","ebb","bbb","fb","cb","gb","db","ab","eb","bb","f","c","g","d","a","e","b","f#","c#","g#","d#","a#","e#","b#","f##","c##","g##","d##","a##","e##","b##"];
    var cursor = curScore.newCursor();
    
	var chordnames = new Array(1024); 
		// chordnames
		// O: chordname
		// 1: tick
		// 2: head
		// 3: tail
		for(var i=0; i<1024; i++) chordnames[i]=new Array();
		var count = 0;
		for (var track = 0; track < /*curScore.ntracks*/ 1; ++track) {
			var segment = curScore.firstSegment();
            count = collectchordnames(segment, chordnames,count);
            
		}
    // melnotes
	// O: notename
	// 1: pitch
    // 2: tpc
    // 3: duration
    // 4: chord symbol
    var melnotes = new Array(2048); 
	for(var i=0; i<2048; i++) melnotes[i]=new Array();
	var mcount = 0;
        
    cursor.rewind(0); // beginning of score
    cursor.staffIdx = 0;
    while (cursor.segment) {
        
        if (cursor.element && cursor.element.type === Element.CHORD) {
            var notes = cursor.element.notes;
            var name;
            // collect top note only
            
            var i = notes.length -1;
            melnotes[mcount][0] = tpc2note[notes[i].tpc];
            melnotes[mcount][1] = notes[i].pitch;
            melnotes[mcount][2] = notes[i].tpc;
            melnotes[mcount][3] = cursor.tick;
            melnotes[mcount][4] = "";
            for (var c = 0; c < count; c++){
                if (chordnames[c][1] == cursor.tick) melnotes[mcount][4] = chordnames[c][0];
            }
            console.log (melnotes[mcount][0] + " " + melnotes[mcount][1] + " " + melnotes[mcount][2] + " " + melnotes[mcount][3])
            mcount++;
        }// end if CHORD 
        cursor.next();
        
    } // end while segment
    melnotes[mcount][3] = cursor.tick + 1920;
    
    //set adjusted duration
    for (var c = 0; c < mcount; c++) {
        duration = melnotes[c+1][3] - melnotes[c][3];
        if (melnotes[c][3] % 1440 == 0) duration += 480;
        melnotes[c][3] = duration;
        console.log (melnotes[c][0] + " " + melnotes[c][1] + " " + melnotes[c][2] + " " + melnotes[c][3])
     }   

    curScore.startCmd();
    cmd("next-score");
    curScore.endCmd();

    var cursor = curScore.newCursor()
    if (typeof curScore === 'undefined') {
        console.log ("All done");
        Qt.quit();
    }
    cursor.rewind(0); // beginning of score
    cursor.staffIdx = 0;
    var duration = 0;
    for (var c = 0; c < mcount; c++) {
        if(melnotes[c][4] != ""){
            var harmony = newElement(Element.HARMONY);
            harmony.text = melnotes[c][4];
            cursor.add(harmony);
        }
        var cur_time=cursor.tick;
        duration = melnotes[c][3];
        if (duration == 1200){

            duration = 960;
            cursor.setDuration(duration, 1920);
            cursor.addNote(melnotes[c][1]); //add 1st note
            //addnote moves the cursor to the next segment
            var next_time=cursor.tick;
            console.log ("na add notes ", melnotes[c][0] + " " + melnotes[c][1] + " " + melnotes[c][2] + " " + cursor.tick)
            //rewind to this note
            setCursorToTime(cursor, cur_time); 
            var note = cursor.element.notes[0];
            // now set the tpc
            note.tpc = melnotes[c][2];
            //move the cursor forward to where it was
            setCursorToTime(cursor, next_time);
            //setCursorToTime(cursor, cur_time + duration);
            duration = 240;
        }
        cursor.setDuration(duration, 1920);
        cursor.addNote(melnotes[c][1]); //add 1st note
        //addnote moves the cursor to the next segment
        var next_time=cursor.tick;
        console.log ("na add notes ", melnotes[c][0] + " " + melnotes[c][1] + " " + melnotes[c][2] + " " + cursor.tick)
        //rewind to this note
        setCursorToTime(cursor, cur_time); 
		var note = cursor.element.notes[0];
        // now set the tpc
		note.tpc = melnotes[c][2];
        //move the cursor forward to where it was
        setCursorToTime(cursor, next_time);
        //setCursorToTime(cursor, cur_time + duration);
    }        

	smartQuit();
	}// end on Run
}
